@@ -1,7 +1,7 @@
 /******************************************************************************
  *
  * Project:  PROJ
- * Purpose:  ISO19111:2018 implementation
+ * Purpose:  ISO19111:2019 implementation
  * Author:   Even Rouault <even dot rouault at spatialys dot com>
  *
  ******************************************************************************
@@ -68,6 +68,12 @@ using BoundCRSPtr = std::shared_ptr<BoundCRS>;
 /** Non-null shared pointer of BoundCRS */
 using BoundCRSNNPtr = util::nn<BoundCRSPtr>;
 
+class CompoundCRS;
+/** Shared pointer of CompoundCRS */
+using CompoundCRSPtr = std::shared_ptr<CompoundCRS>;
+/** Non-null shared pointer of CompoundCRS */
+using CompoundCRSNNPtr = util::nn<CompoundCRSPtr>;
+
 // ---------------------------------------------------------------------------
 
 class CRS;
@@ -79,9 +85,10 @@ using CRSNNPtr = util::nn<CRSPtr>;
 /** \brief Abstract class modelling a coordinate reference system which is
  * usually single but may be compound.
  *
- * \remark Implements CRS from \ref ISO_19111_2018
+ * \remark Implements CRS from \ref ISO_19111_2019
  */
-class PROJ_GCC_DLL CRS : public common::ObjectUsage {
+class PROJ_GCC_DLL CRS : public common::ObjectUsage,
+                         public io::IJSONExportable {
   public:
     //! @cond Doxygen_Suppress
     PROJ_DLL ~CRS() override;
@@ -106,6 +113,13 @@ class PROJ_GCC_DLL CRS : public common::ObjectUsage {
     PROJ_DLL std::list<CRSNNPtr>
     getNonDeprecated(const io::DatabaseContextNNPtr &dbContext) const;
 
+    PROJ_DLL CRSNNPtr
+    promoteTo3D(const std::string &newName,
+                const io::DatabaseContextPtr &dbContext) const;
+
+    PROJ_DLL CRSNNPtr demoteTo2D(const std::string &newName,
+                                 const io::DatabaseContextPtr &dbContext) const;
+
     PROJ_PRIVATE :
         //! @cond Doxygen_Suppress
         PROJ_INTERNAL const GeodeticCRS *
@@ -126,6 +140,21 @@ class PROJ_GCC_DLL CRS : public common::ObjectUsage {
     PROJ_FOR_TEST CRSNNPtr
     alterCSLinearUnit(const common::UnitOfMeasure &unit) const;
 
+    PROJ_INTERNAL bool mustAxisOrderBeSwitchedForVisualization() const;
+
+    PROJ_INTERNAL CRSNNPtr normalizeForVisualization() const;
+
+    PROJ_INTERNAL CRSNNPtr allowNonConformantWKT1Export() const;
+
+    PROJ_INTERNAL CRSNNPtr
+    attachOriginalCompoundCRS(const CompoundCRSNNPtr &compoundCRS) const;
+
+    PROJ_INTERNAL CRSNNPtr promoteTo3D(
+        const std::string &newName, const io::DatabaseContextPtr &dbContext,
+        const cs::CoordinateSystemAxisNNPtr &verticalAxisIfNotAlreadyPresent)
+        const;
+
+    PROJ_INTERNAL bool hasImplicitCS() const;
     //! @endcond
 
   protected:
@@ -149,7 +178,7 @@ class PROJ_GCC_DLL CRS : public common::ObjectUsage {
  * one Coordinate System and either one datum::Datum or one
  * datum::DatumEnsemble.
  *
- * \remark Implements SingleCRS from \ref ISO_19111_2018
+ * \remark Implements SingleCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL SingleCRS : public CRS {
   public:
@@ -166,7 +195,10 @@ class PROJ_GCC_DLL SingleCRS : public CRS {
         PROJ_INTERNAL void
         exportDatumOrDatumEnsembleToWkt(io::WKTFormatter *formatter)
             const; // throw(io::FormattingException)
-                   //! @endcond
+
+    PROJ_INTERNAL const datum::DatumNNPtr
+    datumNonNull(const io::DatabaseContextPtr &dbContext) const;
+    //! @endcond
 
   protected:
     PROJ_INTERNAL SingleCRS(const datum::DatumPtr &datumIn,
@@ -177,7 +209,8 @@ class PROJ_GCC_DLL SingleCRS : public CRS {
     PROJ_INTERNAL bool
     baseIsEquivalentTo(const util::IComparable *other,
                        util::IComparable::Criterion criterion =
-                           util::IComparable::Criterion::STRICT) const;
+                           util::IComparable::Criterion::STRICT,
+                       const io::DatabaseContextPtr &dbContext = nullptr) const;
 
   private:
     PROJ_OPAQUE_PRIVATE_DATA
@@ -204,7 +237,7 @@ using GeodeticCRSNNPtr = util::nn<GeodeticCRSPtr>;
  * association to a velocity model then the geodetic CRS is dynamic, else it
  * is static.
  *
- * \remark Implements GeodeticCRS from \ref ISO_19111_2018
+ * \remark Implements GeodeticCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL GeodeticCRS : virtual public SingleCRS,
                                  public io::IPROJStringExportable {
@@ -260,6 +293,9 @@ class PROJ_GCC_DLL GeodeticCRS : virtual public SingleCRS,
         PROJ_INTERNAL void
         addDatumInfoToPROJString(io::PROJStringFormatter *formatter) const;
 
+    PROJ_INTERNAL const datum::GeodeticReferenceFrameNNPtr
+    datumNonNull(const io::DatabaseContextPtr &dbContext) const;
+
     PROJ_INTERNAL void addGeocentricUnitConversionIntoPROJString(
         io::PROJStringFormatter *formatter) const;
 
@@ -269,10 +305,14 @@ class PROJ_GCC_DLL GeodeticCRS : virtual public SingleCRS,
     PROJ_INTERNAL void _exportToPROJString(io::PROJStringFormatter *formatter)
         const override; // throw(FormattingException)
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
+
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
     //! @endcond
 
@@ -312,7 +352,7 @@ class PROJ_GCC_DLL GeodeticCRS : virtual public SingleCRS,
  * association to a velocity model then the geodetic CRS is dynamic, else it is
  * static.
  *
- * \remark Implements GeographicCRS from \ref ISO_19111_2018
+ * \remark Implements GeographicCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL GeographicCRS : public GeodeticCRS {
   public:
@@ -333,6 +373,10 @@ class PROJ_GCC_DLL GeographicCRS : public GeodeticCRS {
            const datum::DatumEnsemblePtr &datumEnsemble,
            const cs::EllipsoidalCSNNPtr &cs);
 
+    PROJ_DLL GeographicCRSNNPtr
+    demoteTo2D(const std::string &newName,
+               const io::DatabaseContextPtr &dbContext) const;
+
     PROJ_DLL static const GeographicCRSNNPtr EPSG_4267; // NAD27
     PROJ_DLL static const GeographicCRSNNPtr EPSG_4269; // NAD83
     PROJ_DLL static const GeographicCRSNNPtr EPSG_4326; // WGS 84 2D
@@ -349,13 +393,18 @@ class PROJ_GCC_DLL GeographicCRS : public GeodeticCRS {
     PROJ_INTERNAL void _exportToPROJString(io::PROJStringFormatter *formatter)
         const override; // throw(FormattingException)
 
-    PROJ_DLL bool
-    is2DPartOf3D(util::nn<const GeographicCRS *> other) PROJ_PURE_DECL;
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_DLL bool is2DPartOf3D(
+        util::nn<const GeographicCRS *> other,
+        const io::DatabaseContextPtr &dbContext = nullptr) PROJ_PURE_DECL;
+
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
     //! @endcond
 
@@ -398,7 +447,7 @@ class PROJ_GCC_DLL GeographicCRS : public GeodeticCRS {
  * reference system. They exist only as an inseparable part of a 3D coordinate
  * tuple defined in a geographic 3D coordinate reference system.
  *
- * \remark Implements VerticalCRS from \ref ISO_19111_2018
+ * \remark Implements VerticalCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL VerticalCRS : virtual public SingleCRS,
                                  public io::IPROJStringExportable {
@@ -433,13 +482,23 @@ class PROJ_GCC_DLL VerticalCRS : virtual public SingleCRS,
         PROJ_INTERNAL void
         addLinearUnitConvert(io::PROJStringFormatter *formatter) const;
 
+    PROJ_INTERNAL const datum::VerticalReferenceFrameNNPtr
+    datumNonNull(const io::DatabaseContextPtr &dbContext) const;
+
     PROJ_INTERNAL void _exportToWKT(io::WKTFormatter *formatter)
         const override; // throw(io::FormattingException)
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL void _exportToPROJString(io::PROJStringFormatter *formatter)
+        const override; // throw(FormattingException)
+
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
+
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
     //! @endcond
 
@@ -449,9 +508,6 @@ class PROJ_GCC_DLL VerticalCRS : virtual public SingleCRS,
                               const cs::VerticalCSNNPtr &csIn);
     PROJ_INTERNAL VerticalCRS(const VerticalCRS &other);
 
-    PROJ_INTERNAL void _exportToPROJString(io::PROJStringFormatter *formatter)
-        const override; // throw(FormattingException)
-
     PROJ_INTERNAL std::list<std::pair<CRSNNPtr, int>>
     _identify(const io::AuthorityFactoryPtr &authorityFactory) const override;
 
@@ -476,7 +532,7 @@ class PROJ_GCC_DLL VerticalCRS : virtual public SingleCRS,
  * coordinate reference system is implemented using the parameters and
  * formula(s) specified in the definition of the coordinate conversion.
  *
- * \remark Implements DerivedCRS from \ref ISO_19111_2018
+ * \remark Implements DerivedCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL DerivedCRS : virtual public SingleCRS {
   public:
@@ -489,8 +545,15 @@ class PROJ_GCC_DLL DerivedCRS : virtual public SingleCRS {
 
     PROJ_PRIVATE :
         //! @cond Doxygen_Suppress
+
+        // Use this method with extreme care ! It should never be used
+        // to recreate a new Derived/ProjectedCRS !
         PROJ_INTERNAL const operation::ConversionNNPtr &
         derivingConversionRef() PROJ_PURE_DECL;
+
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
+
     //! @endcond
 
   protected:
@@ -503,13 +566,16 @@ class PROJ_GCC_DLL DerivedCRS : virtual public SingleCRS {
     PROJ_INTERNAL void setDerivingConversionCRS();
 
     PROJ_INTERNAL void baseExportToWKT(
-        io::WKTFormatter *&formatter, const std::string &keyword,
+        io::WKTFormatter *formatter, const std::string &keyword,
         const std::string &baseKeyword) const; // throw(FormattingException)
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
+
+    PROJ_INTERNAL virtual const char *className() const = 0;
 
   private:
     PROJ_OPAQUE_PRIVATE_DATA
@@ -539,7 +605,7 @@ using ProjectedCRSNNPtr = util::nn<ProjectedCRSPtr>;
  * height is passed through unchanged and forms the vertical axis of the
  * projected CRS's Cartesian coordinate system.
  *
- * \remark Implements ProjectedCRS from \ref ISO_19111_2018
+ * \remark Implements ProjectedCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL ProjectedCRS final : public DerivedCRS,
                                         public io::IPROJStringExportable {
@@ -560,6 +626,10 @@ class PROJ_GCC_DLL ProjectedCRS final : public DerivedCRS,
     PROJ_DLL std::list<std::pair<ProjectedCRSNNPtr, int>>
     identify(const io::AuthorityFactoryPtr &authorityFactory) const;
 
+    PROJ_DLL ProjectedCRSNNPtr
+    demoteTo2D(const std::string &newName,
+               const io::DatabaseContextPtr &dbContext) const;
+
     PROJ_PRIVATE :
         //! @cond Doxygen_Suppress
         PROJ_INTERNAL void
@@ -569,6 +639,9 @@ class PROJ_GCC_DLL ProjectedCRS final : public DerivedCRS,
     PROJ_INTERNAL void _exportToWKT(io::WKTFormatter *formatter)
         const override; // throw(io::FormattingException)
 
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
+
     PROJ_FOR_TEST ProjectedCRSNNPtr alterParametersLinearUnit(
         const common::UnitOfMeasure &unit, bool convertToNewUnit) const;
 
@@ -584,14 +657,19 @@ class PROJ_GCC_DLL ProjectedCRS final : public DerivedCRS,
     PROJ_INTERNAL void _exportToPROJString(io::PROJStringFormatter *formatter)
         const override; // throw(FormattingException)
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
     PROJ_INTERNAL std::list<std::pair<CRSNNPtr, int>>
     _identify(const io::AuthorityFactoryPtr &authorityFactory) const override;
 
+    PROJ_INTERNAL const char *className() const override {
+        return "ProjectedCRS";
+    }
+
     INLINED_MAKE_SHARED
 
     PROJ_INTERNAL CRSNNPtr _shallowClone() const override;
@@ -612,7 +690,7 @@ using TemporalCRSNNPtr = util::nn<TemporalCRSPtr>;
 /** \brief A coordinate reference system associated with a temporal datum and a
  * one-dimensional temporal coordinate system.
  *
- * \remark Implements TemporalCRS from \ref ISO_19111_2018
+ * \remark Implements TemporalCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL TemporalCRS : virtual public SingleCRS {
   public:
@@ -632,7 +710,11 @@ class PROJ_GCC_DLL TemporalCRS : virtual public SingleCRS {
     //! @cond Doxygen_Suppress
     PROJ_INTERNAL void _exportToWKT(io::WKTFormatter *formatter)
         const override; // throw(io::FormattingException)
-                        //! @endcond
+
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
+
+    //! @endcond
 
   protected:
     PROJ_INTERNAL TemporalCRS(const datum::TemporalDatumNNPtr &datumIn,
@@ -643,10 +725,11 @@ class PROJ_GCC_DLL TemporalCRS : virtual public SingleCRS {
 
     PROJ_INTERNAL CRSNNPtr _shallowClone() const override;
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
   private:
     PROJ_OPAQUE_PRIVATE_DATA
@@ -671,7 +754,7 @@ using EngineeringCRSNNPtr = util::nn<EngineeringCRSPtr>;
  * In \ref WKT2, it maps to a ENGINEERINGCRS / ENGCRS keyword. In \ref WKT1,
  * it maps to a LOCAL_CS keyword.
  *
- * \remark Implements EngineeringCRS from \ref ISO_19111_2018
+ * \remark Implements EngineeringCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL EngineeringCRS : virtual public SingleCRS {
   public:
@@ -689,7 +772,11 @@ class PROJ_GCC_DLL EngineeringCRS : virtual public SingleCRS {
     //! @cond Doxygen_Suppress
     PROJ_INTERNAL void _exportToWKT(io::WKTFormatter *formatter)
         const override; // throw(io::FormattingException)
-                        //! @endcond
+
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
+
+    //! @endcond
 
   protected:
     PROJ_INTERNAL EngineeringCRS(const datum::EngineeringDatumNNPtr &datumIn,
@@ -698,10 +785,11 @@ class PROJ_GCC_DLL EngineeringCRS : virtual public SingleCRS {
 
     PROJ_INTERNAL CRSNNPtr _shallowClone() const override;
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
     INLINED_MAKE_SHARED
 
@@ -725,7 +813,7 @@ using ParametricCRSNNPtr = util::nn<ParametricCRSPtr>;
  * without geodetic corrections, or on moving platforms such as road vehicles
  * vessels, aircraft or spacecraft, or as the internal CRS of an image.
  *
- * \remark Implements ParametricCRS from \ref ISO_19111_2018
+ * \remark Implements ParametricCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL ParametricCRS : virtual public SingleCRS {
   public:
@@ -745,7 +833,11 @@ class PROJ_GCC_DLL ParametricCRS : virtual public SingleCRS {
     //! @cond Doxygen_Suppress
     PROJ_INTERNAL void _exportToWKT(io::WKTFormatter *formatter)
         const override; // throw(io::FormattingException)
-                        //! @endcond
+
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
+
+    //! @endcond
 
   protected:
     PROJ_INTERNAL ParametricCRS(const datum::ParametricDatumNNPtr &datumIn,
@@ -754,10 +846,11 @@ class PROJ_GCC_DLL ParametricCRS : virtual public SingleCRS {
 
     PROJ_INTERNAL CRSNNPtr _shallowClone() const override;
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
     INLINED_MAKE_SHARED
 
@@ -768,11 +861,21 @@ class PROJ_GCC_DLL ParametricCRS : virtual public SingleCRS {
 
 // ---------------------------------------------------------------------------
 
-class CompoundCRS;
-/** Shared pointer of CompoundCRS */
-using CompoundCRSPtr = std::shared_ptr<CompoundCRS>;
-/** Non-null shared pointer of CompoundCRS */
-using CompoundCRSNNPtr = util::nn<CompoundCRSPtr>;
+/** \brief Exception thrown when attempting to create an invalid compound CRS
+ */
+class PROJ_GCC_DLL InvalidCompoundCRSException : public util::Exception {
+  public:
+    //! @cond Doxygen_Suppress
+    PROJ_INTERNAL explicit InvalidCompoundCRSException(const char *message);
+    PROJ_INTERNAL explicit InvalidCompoundCRSException(
+        const std::string &message);
+    PROJ_DLL
+    InvalidCompoundCRSException(const InvalidCompoundCRSException &other);
+    PROJ_DLL ~InvalidCompoundCRSException() override;
+    //! @endcond
+};
+
+// ---------------------------------------------------------------------------
 
 /** \brief A coordinate reference system describing the position of points
  * through two or more independent single coordinate reference systems.
@@ -781,11 +884,11 @@ using CompoundCRSNNPtr = util::nn<CompoundCRSPtr>;
  * if coordinate values in one cannot be converted or transformed into
  * coordinate values in the other.
  *
- * \note As a departure to \ref ISO_19111_2018, we allow to build a CompoundCRS
- * from CRS objects, whereas ISO19111:2018 restricts the components to
+ * \note As a departure to \ref ISO_19111_2019, we allow to build a CompoundCRS
+ * from CRS objects, whereas ISO19111:2019 restricts the components to
  * SingleCRS.
  *
- * \remark Implements CompoundCRS from \ref ISO_19111_2018
+ * \remark Implements CompoundCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL CompoundCRS final : public CRS,
                                        public io::IPROJStringExportable {
@@ -807,7 +910,16 @@ class PROJ_GCC_DLL CompoundCRS final : public CRS,
 
     PROJ_DLL static CompoundCRSNNPtr
     create(const util::PropertyMap &properties,
-           const std::vector<CRSNNPtr> &components);
+           const std::vector<CRSNNPtr>
+               &components); // throw InvalidCompoundCRSException
+
+    //! @cond Doxygen_Suppress
+    PROJ_INTERNAL static CRSNNPtr
+    createLax(const util::PropertyMap &properties,
+              const std::vector<CRSNNPtr> &components,
+              const io::DatabaseContextPtr
+                  &dbContext); // throw InvalidCompoundCRSException
+                               //! @endcond
 
   protected:
     // relaxed: standard say SingleCRSNNPtr
@@ -817,12 +929,16 @@ class PROJ_GCC_DLL CompoundCRS final : public CRS,
     PROJ_INTERNAL void _exportToPROJString(io::PROJStringFormatter *formatter)
         const override; // throw(FormattingException)
 
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
+
     PROJ_INTERNAL CRSNNPtr _shallowClone() const override;
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
     PROJ_INTERNAL std::list<std::pair<CRSNNPtr, int>>
     _identify(const io::AuthorityFactoryPtr &authorityFactory) const override;
@@ -857,7 +973,7 @@ class PROJ_GCC_DLL CompoundCRS final : public CRS,
  * present in \ref WKT1.
  *
  * \note Contrary to other CRS classes of this package, there is no
- * \ref ISO_19111_2018 modelling of a BoundCRS.
+ * \ref ISO_19111_2019 modelling of a BoundCRS.
  *
  * \remark Implements BoundCRS from \ref WKT2
  */
@@ -902,10 +1018,14 @@ class PROJ_GCC_DLL BoundCRS final : public CRS,
     PROJ_INTERNAL void _exportToPROJString(io::PROJStringFormatter *formatter)
         const override; // throw(FormattingException)
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
+
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
     PROJ_INTERNAL BoundCRSNNPtr shallowCloneAsBoundCRS() const;
     PROJ_INTERNAL bool isTOWGS84Compatible() const;
@@ -935,7 +1055,7 @@ using DerivedGeodeticCRSNNPtr = util::nn<DerivedGeodeticCRSPtr>;
  * inheriting a geodetic reference frame, and associated with a 3D Cartesian
  * or spherical coordinate system.
  *
- * \remark Implements DerivedGeodeticCRS from \ref ISO_19111_2018
+ * \remark Implements DerivedGeodeticCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL DerivedGeodeticCRS final : public GeodeticCRS,
                                               public DerivedCRS {
@@ -961,7 +1081,13 @@ class PROJ_GCC_DLL DerivedGeodeticCRS final : public GeodeticCRS,
     //! @cond Doxygen_Suppress
     void _exportToWKT(io::WKTFormatter *formatter)
         const override; // throw(io::FormattingException)
-                        //! @endcond
+
+    PROJ_INTERNAL void
+    _exportToJSON(io::JSONFormatter *formatter) const override {
+        return DerivedCRS::_exportToJSON(formatter);
+    }
+
+    //! @endcond
 
   protected:
     PROJ_INTERNAL
@@ -976,10 +1102,11 @@ class PROJ_GCC_DLL DerivedGeodeticCRS final : public GeodeticCRS,
 
     PROJ_INTERNAL CRSNNPtr _shallowClone() const override;
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
     PROJ_INTERNAL std::list<std::pair<CRSNNPtr, int>>
     _identify(const io::AuthorityFactoryPtr &authorityFactory) const override;
@@ -988,6 +1115,10 @@ class PROJ_GCC_DLL DerivedGeodeticCRS final : public GeodeticCRS,
     PROJ_INTERNAL void _exportToPROJString(io::PROJStringFormatter *formatter)
         const override; // throw(FormattingException)
 
+    PROJ_INTERNAL const char *className() const override {
+        return "DerivedGeodeticCRS";
+    }
+
     INLINED_MAKE_SHARED
 
   private:
@@ -1010,7 +1141,7 @@ using DerivedGeographicCRSNNPtr = util::nn<DerivedGeographicCRSPtr>;
  * A derived geographic CRS can be based on a geodetic CRS only if that
  * geodetic CRS definition includes an ellipsoid.
  *
- * \remark Implements DerivedGeographicCRS from \ref ISO_19111_2018
+ * \remark Implements DerivedGeographicCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL DerivedGeographicCRS final : public GeographicCRS,
                                                 public DerivedCRS {
@@ -1030,7 +1161,13 @@ class PROJ_GCC_DLL DerivedGeographicCRS final : public GeographicCRS,
     //! @cond Doxygen_Suppress
     PROJ_INTERNAL void _exportToWKT(io::WKTFormatter *formatter)
         const override; // throw(io::FormattingException)
-                        //! @endcond
+
+    PROJ_INTERNAL void
+    _exportToJSON(io::JSONFormatter *formatter) const override {
+        return DerivedCRS::_exportToJSON(formatter);
+    }
+
+    //! @endcond
 
   protected:
     PROJ_INTERNAL
@@ -1041,14 +1178,19 @@ class PROJ_GCC_DLL DerivedGeographicCRS final : public GeographicCRS,
 
     PROJ_INTERNAL CRSNNPtr _shallowClone() const override;
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
     PROJ_INTERNAL std::list<std::pair<CRSNNPtr, int>>
     _identify(const io::AuthorityFactoryPtr &authorityFactory) const override;
 
+    PROJ_INTERNAL const char *className() const override {
+        return "DerivedGeographicCRS";
+    }
+
     // cppcheck-suppress functionStatic
     PROJ_INTERNAL void _exportToPROJString(io::PROJStringFormatter *formatter)
         const override; // throw(FormattingException)
@@ -1075,7 +1217,7 @@ using DerivedProjectedCRSNNPtr = util::nn<DerivedProjectedCRSPtr>;
  *
  * A DerivedProjectedCRS is not a ProjectedCRS.
  *
- * \remark Implements DerivedProjectedCRS from \ref ISO_19111_2018
+ * \remark Implements DerivedProjectedCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL DerivedProjectedCRS final : public DerivedCRS {
   public:
@@ -1105,10 +1247,15 @@ class PROJ_GCC_DLL DerivedProjectedCRS final : public DerivedCRS {
 
     PROJ_INTERNAL CRSNNPtr _shallowClone() const override;
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
+
+    PROJ_INTERNAL const char *className() const override {
+        return "DerivedProjectedCRS";
+    }
 
     INLINED_MAKE_SHARED
 
@@ -1129,7 +1276,7 @@ using DerivedVerticalCRSNNPtr = util::nn<DerivedVerticalCRSPtr>;
  * coordinate reference system as its base CRS, thereby inheriting a vertical
  * reference frame, and a vertical coordinate system.
  *
- * \remark Implements DerivedVerticalCRS from \ref ISO_19111_2018
+ * \remark Implements DerivedVerticalCRS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL DerivedVerticalCRS final : public VerticalCRS,
                                               public DerivedCRS {
@@ -1149,7 +1296,13 @@ class PROJ_GCC_DLL DerivedVerticalCRS final : public VerticalCRS,
     //! @cond Doxygen_Suppress
     PROJ_INTERNAL void _exportToWKT(io::WKTFormatter *formatter)
         const override; // throw(io::FormattingException)
-                        //! @endcond
+
+    PROJ_INTERNAL void
+    _exportToJSON(io::JSONFormatter *formatter) const override {
+        return DerivedCRS::_exportToJSON(formatter);
+    }
+
+    //! @endcond
 
   protected:
     PROJ_INTERNAL
@@ -1160,14 +1313,19 @@ class PROJ_GCC_DLL DerivedVerticalCRS final : public VerticalCRS,
 
     PROJ_INTERNAL CRSNNPtr _shallowClone() const override;
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
     PROJ_INTERNAL std::list<std::pair<CRSNNPtr, int>>
     _identify(const io::AuthorityFactoryPtr &authorityFactory) const override;
 
+    PROJ_INTERNAL const char *className() const override {
+        return "DerivedVerticalCRS";
+    }
+
     // cppcheck-suppress functionStatic
     PROJ_INTERNAL void _exportToPROJString(io::PROJStringFormatter *formatter)
         const override; // throw(FormattingException)
@@ -1205,23 +1363,23 @@ class PROJ_GCC_DLL DerivedCRSTemplate final : public DerivedCRSTraits::BaseType,
     typedef util::nn<std::shared_ptr<CSType>> CSNNPtr;
 
     /** \brief Return the base CRS of a DerivedCRSTemplate.
-    *
-    * @return the base CRS.
-    */
+     *
+     * @return the base CRS.
+     */
     PROJ_DLL const BaseNNPtr baseCRS() const;
 
     /** \brief Instantiate a DerivedCRSTemplate from a base CRS, a deriving
-    * conversion and a cs::CoordinateSystem.
-    *
-    * @param properties See \ref general_properties.
-    * At minimum the name should be defined.
-    * @param baseCRSIn base CRS.
-    * @param derivingConversionIn the deriving conversion from the base CRS to
-    * this
-    * CRS.
-    * @param csIn the coordinate system.
-    * @return new DerivedCRSTemplate.
-    */
+     * conversion and a cs::CoordinateSystem.
+     *
+     * @param properties See \ref general_properties.
+     * At minimum the name should be defined.
+     * @param baseCRSIn base CRS.
+     * @param derivingConversionIn the deriving conversion from the base CRS to
+     * this
+     * CRS.
+     * @param csIn the coordinate system.
+     * @return new DerivedCRSTemplate.
+     */
     PROJ_DLL static NNPtr
     create(const util::PropertyMap &properties, const BaseNNPtr &baseCRSIn,
            const operation::ConversionNNPtr &derivingConversionIn,
@@ -1230,7 +1388,12 @@ class PROJ_GCC_DLL DerivedCRSTemplate final : public DerivedCRSTraits::BaseType,
     //! @cond Doxygen_Suppress
     PROJ_INTERNAL void _exportToWKT(io::WKTFormatter *formatter)
         const override; // throw(io::FormattingException)
-                        //! @endcond
+
+    PROJ_INTERNAL void
+    _exportToJSON(io::JSONFormatter *formatter) const override {
+        return DerivedCRS::_exportToJSON(formatter);
+    }
+    //! @endcond
 
   protected:
     PROJ_INTERNAL
@@ -1241,10 +1404,13 @@ class PROJ_GCC_DLL DerivedCRSTemplate final : public DerivedCRSTraits::BaseType,
 
     PROJ_INTERNAL CRSNNPtr _shallowClone() const override;
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
+
+    PROJ_INTERNAL const char *className() const override;
 
     INLINED_MAKE_SHARED
 
@@ -1266,7 +1432,7 @@ struct PROJ_GCC_DLL DerivedEngineeringCRSTraits {
     inline static const std::string &CRSName();
     inline static const std::string &WKTKeyword();
     inline static const std::string &WKTBaseKeyword();
-    static const bool wkt2_2018_only = true;
+    static const bool wkt2_2019_only = true;
 };
 //! @endcond
 
@@ -1275,7 +1441,7 @@ struct PROJ_GCC_DLL DerivedEngineeringCRSTraits {
  * engineering datum, and is associated with one of the coordinate system
  * types for an EngineeringCRS
  *
- * \remark Implements DerivedEngineeringCRS from \ref ISO_19111_2018
+ * \remark Implements DerivedEngineeringCRS from \ref ISO_19111_2019
  */
 #ifdef DOXYGEN_ENABLED
 class DerivedEngineeringCRS
@@ -1304,7 +1470,7 @@ struct PROJ_GCC_DLL DerivedParametricCRSTraits {
     inline static const std::string &CRSName();
     inline static const std::string &WKTKeyword();
     inline static const std::string &WKTBaseKeyword();
-    static const bool wkt2_2018_only = false;
+    static const bool wkt2_2019_only = false;
 };
 //! @endcond
 
@@ -1312,7 +1478,7 @@ struct PROJ_GCC_DLL DerivedParametricCRSTraits {
  * coordinate reference system as its base CRS, thereby inheriting a parametric
  * datum, and a parametric coordinate system.
  *
- * \remark Implements DerivedParametricCRS from \ref ISO_19111_2018
+ * \remark Implements DerivedParametricCRS from \ref ISO_19111_2019
  */
 #ifdef DOXYGEN_ENABLED
 class DerivedParametricCRS
@@ -1341,7 +1507,7 @@ struct PROJ_GCC_DLL DerivedTemporalCRSTraits {
     inline static const std::string &CRSName();
     inline static const std::string &WKTKeyword();
     inline static const std::string &WKTBaseKeyword();
-    static const bool wkt2_2018_only = false;
+    static const bool wkt2_2019_only = false;
 };
 //! @endcond
 
@@ -1349,7 +1515,7 @@ struct PROJ_GCC_DLL DerivedTemporalCRSTraits {
  * coordinate reference system as its base CRS, thereby inheriting a temporal
  * datum, and a temporal coordinate system.
  *
- * \remark Implements DerivedTemporalCRS from \ref ISO_19111_2018
+ * \remark Implements DerivedTemporalCRS from \ref ISO_19111_2019
  */
 #ifdef DOXYGEN_ENABLED
 class DerivedTemporalCRS : public DerivedCRSTemplate<DerivedTemporalCRSTraits> {
